(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms')) :
    typeof define === 'function' && define.amd ? define('ngx-input-mask', ['exports', '@angular/core', '@angular/forms'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-input-mask'] = {}, global.ng.core, global.ng.forms));
}(this, (function (exports, core, forms) { 'use strict';

    var MASK_TEMPLATES = new core.InjectionToken('MaskTemplates');
    var DEFAULT_MASK_TEMPLATES = {
        '0': new RegExp('[0-9]'),
        '_': new RegExp('[a-z]', 'i'),
        'A': new RegExp('[A-Z]'),
        'a': new RegExp('[a-z]'),
    };

    var InputMaskDirective = /** @class */ (function () {
        function InputMaskDirective(_elementRef, _ngControl, _templates) {
            this._elementRef = _elementRef;
            this._ngControl = _ngControl;
            this._templates = _templates;
            this._inputMask = [];
        }
        InputMaskDirective.prototype.ngOnInit = function () {
            this._element = this._elementRef.nativeElement;
            this._updateInputMask();
        };
        InputMaskDirective.prototype.ngOnChanges = function (changes) {
            if (changes.inputMask) {
                this._updateInputMask();
            }
        };
        InputMaskDirective.prototype.onKeydown = function (event) {
            var _a, _b, _c;
            if (event instanceof KeyboardEvent && (event.ctrlKey || event.altKey || event.key.length > 1)) {
                return;
            }
            var putValue = event instanceof KeyboardEvent ? event.key : (_a = event.clipboardData) === null || _a === void 0 ? void 0 : _a.getData('text');
            var el = this._element;
            if (!putValue || !el) {
                return;
            }
            event.preventDefault();
            try {
                var currentValue = el.value;
                var startLength = currentValue.length;
                var selectionStart = (_b = el.selectionStart) !== null && _b !== void 0 ? _b : startLength;
                // console.log(event, currentValue, startLength, selectionStart, putValue);
                if (selectionStart === currentValue.length) {
                    this._put(currentValue, putValue, '');
                }
                else {
                    var selectionEnd = (_c = el.selectionEnd) !== null && _c !== void 0 ? _c : startLength;
                    var currentValueStart = currentValue.substring(0, selectionStart);
                    var currentValueEnd = currentValue.substring(selectionEnd);
                    var pos = this._put(currentValueStart, putValue, currentValueEnd);
                    try {
                        el.focus();
                        el.setSelectionRange(pos, pos);
                    }
                    catch (err) {
                        console.error(err);
                    }
                }
            }
            catch (err) {
                alert(err);
            }
        };
        InputMaskDirective.prototype._put = function (currentValueStart, putValue, currentValueEnd) {
            var currentValue = currentValueStart;
            var startLength = currentValue.length;
            if (!this._ngControl.control) {
                return startLength;
            }
            var inputMask = this._inputMask;
            var j = startLength;
            for (var i = 0, a = putValue, s = a[i], l = a.length; i < l; s = a[++i]) {
                var startIndex = currentValue.length;
                if (startIndex > inputMask.length) {
                    break;
                }
                j = startIndex;
                var m = inputMask[j];
                while (typeof m === 'string') {
                    currentValue += m;
                    if (m === s) {
                        break;
                    }
                    m = inputMask[++j];
                }
                if (typeof m === 'string') {
                    continue;
                }
                if (m === null || m === void 0 ? void 0 : m.test(s)) {
                    currentValue += s;
                }
            }
            var pos = j + 1;
            if (currentValueEnd) {
                for (var i = 0, a = currentValueEnd, s = a[i], l = a.length; i < l; s = a[++i]) {
                    var startIndex = currentValue.length;
                    if (startIndex > inputMask.length) {
                        break;
                    }
                    j = startIndex;
                    var m = inputMask[j];
                    while (typeof m === 'string') {
                        currentValue += m;
                        if (m === s) {
                            break;
                        }
                        m = inputMask[++j];
                    }
                    if (typeof m === 'string') {
                        continue;
                    }
                    if (m === null || m === void 0 ? void 0 : m.test(s)) {
                        currentValue += s;
                    }
                }
            }
            if (startLength < currentValue.length) {
                this._ngControl.control.setValue(currentValue);
            }
            return pos;
        };
        InputMaskDirective.prototype._updateInputMask = function () {
            var _this = this;
            this._inputMask = this.inputMask
                .split('')
                .reduce(function (p, n, i, a) {
                var n1 = a[i + 1];
                if (n === '\\') {
                    if (n1) {
                        p.push(n + n1);
                        a.splice(i + 1, 1);
                    }
                }
                else {
                    p.push(n);
                }
                return p;
            }, [])
                .map(function (s) { return s.length === 2 && s.startsWith('\\') ? s.charAt(1) : _this._templates[s] || s; });
        };
        return InputMaskDirective;
    }());
    InputMaskDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: 'input[inputMask][formControlName],input[inputMask][formControl],input[inputMask][ngModel]',
                },] }
    ];
    InputMaskDirective.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: forms.NgControl },
        { type: undefined, decorators: [{ type: core.Inject, args: [MASK_TEMPLATES,] }] }
    ]; };
    InputMaskDirective.propDecorators = {
        inputMask: [{ type: core.Input }],
        onKeydown: [{ type: core.HostListener, args: ['keydown', ['$event'],] }, { type: core.HostListener, args: ['paste', ['$event'],] }]
    };

    var ɵ0 = DEFAULT_MASK_TEMPLATES;
    var InputMaskModule = /** @class */ (function () {
        function InputMaskModule() {
        }
        return InputMaskModule;
    }());
    InputMaskModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [forms.ReactiveFormsModule],
                    exports: [InputMaskDirective],
                    declarations: [InputMaskDirective],
                    providers: [{ provide: MASK_TEMPLATES, useValue: ɵ0 }],
                },] }
    ];

    /*
     * Public API Surface of ngx-input-mask
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.DEFAULT_MASK_TEMPLATES = DEFAULT_MASK_TEMPLATES;
    exports.InputMaskDirective = InputMaskDirective;
    exports.InputMaskModule = InputMaskModule;
    exports.MASK_TEMPLATES = MASK_TEMPLATES;
    exports.ɵ0 = ɵ0;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-input-mask.umd.js.map
