import { InjectionToken, Directive, ElementRef, Inject, Input, HostListener, NgModule } from '@angular/core';
import { NgControl, ReactiveFormsModule } from '@angular/forms';

const MASK_TEMPLATES = new InjectionToken('MaskTemplates');
const DEFAULT_MASK_TEMPLATES = {
    '0': new RegExp('[0-9]'),
    '_': new RegExp('[a-z]', 'i'),
    'A': new RegExp('[A-Z]'),
    'a': new RegExp('[a-z]'),
};

class InputMaskDirective {
    constructor(_elementRef, _ngControl, _templates) {
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._templates = _templates;
        this._inputMask = [];
    }
    ngOnInit() {
        this._element = this._elementRef.nativeElement;
        this._updateInputMask();
    }
    ngOnChanges(changes) {
        if (changes.inputMask) {
            this._updateInputMask();
        }
    }
    onKeydown(event) {
        var _a, _b, _c;
        if (event instanceof KeyboardEvent && (event.ctrlKey || event.altKey || event.key.length > 1)) {
            return;
        }
        const putValue = event instanceof KeyboardEvent ? event.key : (_a = event.clipboardData) === null || _a === void 0 ? void 0 : _a.getData('text');
        const el = this._element;
        if (!putValue || !el) {
            return;
        }
        event.preventDefault();
        try {
            const currentValue = el.value;
            const startLength = currentValue.length;
            const selectionStart = (_b = el.selectionStart) !== null && _b !== void 0 ? _b : startLength;
            // console.log(event, currentValue, startLength, selectionStart, putValue);
            if (selectionStart === currentValue.length) {
                this._put(currentValue, putValue, '');
            }
            else {
                const selectionEnd = (_c = el.selectionEnd) !== null && _c !== void 0 ? _c : startLength;
                const currentValueStart = currentValue.substring(0, selectionStart);
                const currentValueEnd = currentValue.substring(selectionEnd);
                const pos = this._put(currentValueStart, putValue, currentValueEnd);
                try {
                    el.focus();
                    el.setSelectionRange(pos, pos);
                }
                catch (err) {
                    console.error(err);
                }
            }
        }
        catch (err) {
            alert(err);
        }
    }
    _put(currentValueStart, putValue, currentValueEnd) {
        let currentValue = currentValueStart;
        const startLength = currentValue.length;
        if (!this._ngControl.control) {
            return startLength;
        }
        const inputMask = this._inputMask;
        let j = startLength;
        for (let i = 0, a = putValue, s = a[i], l = a.length; i < l; s = a[++i]) {
            const startIndex = currentValue.length;
            if (startIndex > inputMask.length) {
                break;
            }
            j = startIndex;
            let m = inputMask[j];
            while (typeof m === 'string') {
                currentValue += m;
                if (m === s) {
                    break;
                }
                m = inputMask[++j];
            }
            if (typeof m === 'string') {
                continue;
            }
            if (m === null || m === void 0 ? void 0 : m.test(s)) {
                currentValue += s;
            }
        }
        const pos = j + 1;
        if (currentValueEnd) {
            for (let i = 0, a = currentValueEnd, s = a[i], l = a.length; i < l; s = a[++i]) {
                const startIndex = currentValue.length;
                if (startIndex > inputMask.length) {
                    break;
                }
                j = startIndex;
                let m = inputMask[j];
                while (typeof m === 'string') {
                    currentValue += m;
                    if (m === s) {
                        break;
                    }
                    m = inputMask[++j];
                }
                if (typeof m === 'string') {
                    continue;
                }
                if (m === null || m === void 0 ? void 0 : m.test(s)) {
                    currentValue += s;
                }
            }
        }
        if (startLength < currentValue.length) {
            this._ngControl.control.setValue(currentValue);
        }
        return pos;
    }
    _updateInputMask() {
        this._inputMask = this.inputMask
            .split('')
            .reduce((p, n, i, a) => {
            const n1 = a[i + 1];
            if (n === '\\') {
                if (n1) {
                    p.push(n + n1);
                    a.splice(i + 1, 1);
                }
            }
            else {
                p.push(n);
            }
            return p;
        }, [])
            .map((s) => s.length === 2 && s.startsWith('\\') ? s.charAt(1) : this._templates[s] || s);
    }
}
InputMaskDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[inputMask][formControlName],input[inputMask][formControl],input[inputMask][ngModel]',
            },] }
];
InputMaskDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgControl },
    { type: undefined, decorators: [{ type: Inject, args: [MASK_TEMPLATES,] }] }
];
InputMaskDirective.propDecorators = {
    inputMask: [{ type: Input }],
    onKeydown: [{ type: HostListener, args: ['keydown', ['$event'],] }, { type: HostListener, args: ['paste', ['$event'],] }]
};

const ɵ0 = DEFAULT_MASK_TEMPLATES;
class InputMaskModule {
}
InputMaskModule.decorators = [
    { type: NgModule, args: [{
                imports: [ReactiveFormsModule],
                exports: [InputMaskDirective],
                declarations: [InputMaskDirective],
                providers: [{ provide: MASK_TEMPLATES, useValue: ɵ0 }],
            },] }
];

/*
 * Public API Surface of ngx-input-mask
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DEFAULT_MASK_TEMPLATES, InputMaskDirective, InputMaskModule, MASK_TEMPLATES, ɵ0 };
//# sourceMappingURL=ngx-input-mask.js.map
